@@ -57,7 +57,7 @@
  *
  * requestobject.c 
  *
- * $Id: requestobject.c,v 1.50 2003/08/04 14:54:46 grisha Exp $
+ * $Id: requestobject.c,v 1.53 2003/10/08 03:48:17 grisha Exp $
  *
  */
 
@@ -498,8 +498,8 @@ static PyObject * req_read(requestobject *self, PyObject *args)
     
     /* Free rbuff if we're done with it */
     if (self->rbuff_pos >= self->rbuff_len && self->rbuff != NULL) {
-	free(self->rbuff);
-	self->rbuff = NULL;
+        free(self->rbuff);
+        self->rbuff = NULL;
     }
 
     if (copied == len)
@@ -618,8 +618,8 @@ static PyObject * req_readline(requestobject *self, PyObject *args)
        somehow? */
     if (self->rbuff_pos >= self->rbuff_len && self->rbuff != NULL)
     {
-	free(self->rbuff);
-	self->rbuff = NULL;
+        free(self->rbuff);
+        self->rbuff = NULL;
     }
 
     /* if got this far, the buffer should be empty, we need to read more */
@@ -649,9 +649,9 @@ static PyObject * req_readline(requestobject *self, PyObject *args)
 
         if (chunk_len == -1) {
 
-	    /* Free rbuff since returning NULL here should end the request */
-	    free(self->rbuff);
-	    self->rbuff = NULL;
+            /* Free rbuff since returning NULL here should end the request */
+            free(self->rbuff);
+            self->rbuff = NULL;
 
             PyErr_SetObject(PyExc_IOError, 
                             PyString_FromString("Client read error (Timeout?)"));
@@ -676,8 +676,8 @@ static PyObject * req_readline(requestobject *self, PyObject *args)
     /* Free rbuff if we're done with it */
     if (self->rbuff_pos >= self->rbuff_len && self->rbuff != NULL)
     {
-	free(self->rbuff);
-	self->rbuff = NULL;
+        free(self->rbuff);
+        self->rbuff = NULL;
     }
 
     /* resize if necessary */
@@ -774,6 +774,48 @@ static PyObject *req_register_cleanup(requestobject *self, PyObject *args)
 
 }
 
+/**
+ ** request.requires(self)
+ **
+ *     Interface to ap_requires()
+ */
+
+static PyObject * req_requires(requestobject *self)
+{
+
+    /* This function returns everything specified after the "requires"
+       as is, without any attempts to parse/organize because
+       "requires" args only need to be grokable by mod_auth if it is
+       authoritative. When AuthAuthoritative is off, anything can
+       follow requires, e.g. "requires role terminator".
+    */
+
+    const apr_array_header_t *reqs_arr = ap_requires(self->request_rec);
+    require_line *reqs;
+    int i, ti = 0;
+
+    PyObject *result;
+
+    if (!reqs_arr) {
+        return Py_BuildValue("()");
+    }
+
+    result = PyTuple_New(reqs_arr->nelts);
+
+    reqs = (require_line *) reqs_arr->elts;
+
+    for (i = 0; i < reqs_arr->nelts; ++i) {
+        if (reqs[i].method_mask & (AP_METHOD_BIT << self->request_rec->method_number)) {
+            PyTuple_SetItem(result, ti++, 
+                            PyString_FromString(reqs[i].requirement));
+        }
+    }
+
+    _PyTuple_Resize(&result, ti);
+
+    return result;
+}
+
 /**
  ** request.send_http_header(request self)
  **
@@ -823,10 +865,11 @@ static PyObject * req_write(requestobject *self, PyObject *args)
     if (len > 0 ) {
 
         Py_BEGIN_ALLOW_THREADS
-        ap_rwrite(buff, len, self->request_rec);
-        rc = ap_rflush(self->request_rec);
+        rc = ap_rwrite(buff, len, self->request_rec);
+        if (rc != -1)
+            rc = ap_rflush(self->request_rec);
         Py_END_ALLOW_THREADS
-            if (rc == EOF) {
+            if (rc == -1) {
                 PyErr_SetString(PyExc_IOError, "Write failed, client closed connection.");
                 return NULL;
             }
@@ -901,6 +944,7 @@ static PyMethodDef request_methods[] = {
     {"readline",              (PyCFunction) req_readline,              METH_VARARGS},
     {"readlines",             (PyCFunction) req_readlines,             METH_VARARGS},
     {"register_cleanup",      (PyCFunction) req_register_cleanup,      METH_VARARGS},
+    {"requires",              (PyCFunction) req_requires,              METH_NOARGS},
     {"send_http_header",      (PyCFunction) req_send_http_header,      METH_NOARGS},
     {"sendfile",              (PyCFunction) req_sendfile,              METH_VARARGS},
     {"set_content_length",    (PyCFunction) req_set_content_length,    METH_VARARGS},
@@ -1088,13 +1132,13 @@ static PyObject *getreq_recmbr_off(requestobject *self, void *name)
     PyMemberDef *md = find_memberdef(request_rec_mbrs, name);
     char *addr = (char *)self->request_rec + md->offset;
     if (sizeof(apr_off_t) == sizeof(LONG_LONG)) {
-	LONG_LONG l = *(LONG_LONG*)addr;
-	return PyLong_FromLongLong(l);
+        LONG_LONG l = *(LONG_LONG*)addr;
+        return PyLong_FromLongLong(l);
     }
     else {
-	/* assume it's long */
-	long l = *(long*)addr;
-	return PyLong_FromLong(l);
+        /* assume it's long */
+        long l = *(long*)addr;
+        return PyLong_FromLong(l);
     }
 }
 
